// A linked list with elements of type 'Interval' supporting queue access methods.
// (The class represents a queue designed for holding intervals prior to processing.)
//
// The intervals of the queue can be accessed in a FIFO (first-in-first-out) manner,
// i.e., the interval that was first inserted by 'addLast' is retrieved first by 'pollFirst'.
// The number of elements of the queue is not limited.
//
public class IntervalQueue {


    // Inserts the specified element into this queue (adds the specified element at the end of
    // the queue).
    // Precondition: interval != null
    public void addLast(Interval interval) {
        nil.addLast(interval);
        size++;
    }

    // Retrieves and removes the head of this queue, or returns 'null' if this queue is empty.
    public Interval pollFirst() {
        Interval temp = nil.pollFirst();
        if (temp != null){
            size--;
        }
        return temp;
    }

    // Retrieves, but does not remove, the head of this queue, or returns 'null' if this queue is
    // empty.
    public Interval peekFirst() {
        return nil.peekFirst();
    }

    // Returns 'true' if and only if this queue contains no elements.
    public boolean isEmpty() {
        return nil.isEmpty();
    }

    // Returns the number of elements in this queue.
    public int size() {
        return size;
    }

    @Override
    // Returns a readable representation with all elements of this queue in brackets, separated by
    // commas. The exact format is shown by the following example with three intervals:
    // [-1.2, 5.0], [7.33, 12.5], [3.3, 8.7]
    // Returns "EMPTY" if the queue is empty.
    public String toString() {
        return nil.toString("");
    }
}

class Node{
    private Node next;
    private Node last;
    private Interval interval;

    public Node(Interval interval){
        this.interval = interval;
        this.next = null;
        this.last = null;
    }

    public Node(Interval interval, Node next, Node last){
        this.interval = interval;
        this.next = next;
        this.last = last;
    }

    public void initNil(){
        this.last = this;
        this.next = this;
    }

    public void addLast(Interval interval){
        Node temp = new Node(interval, this, this.last);
        this.last.next = temp;
        this.last = temp;
    }

    // Retrieves and removes the head of this queue, or returns 'null' if this queue is empty.
    public Interval pollFirst(){
        if (this.next != null){
            Interval temp = this.next.interval;
            this.next = this.next.next;
            return temp;
        } else {
            return null;
        }
    }

    public Interval peekFirst(){
        if (this.next != null){
            return this.next.interval;
        } else {
            return null;
        }
    }

    public boolean isEmpty(){
        return this.next == this;
    }

    // Returns a readable representation with all elements of this queue in brackets, separated by
    // commas. The exact format is shown by the following example with three intervals:
    // [-1.2, 5.0], [7.33, 12.5], [3.3, 8.7]
    // Returns "EMPTY" if the queue is empty.
    public String toString(String text){
        if (this.next == this.last && this.next == this){
            return "EMPTY";
        }
        if (this.next.interval != null){
            return this.next.toString(text + this.next.interval.toString() + ", ");
        } else {
            return text;
        }
    }
}
